--[[
	Copyright (C) 2019 luke < lukaskoschine@gmail.com >

	This file is part of PBI.
	;
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.;

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
]]--

PBI = LibStub("AceAddon-3.0"):NewAddon("PBI", "AceConsole-3.0");

PBI_SIMPLE_NAME = "PetBattleInfo";
PBI_VERSION = "v1.0.3";
PBI_PRINT = PBI_SIMPLE_NAME .. " " .. PBI_VERSION;

local PetBattleInfoSimpleName = PBI_SIMPLE_NAME;
local PetBattleInfoPrint = PBI_PRINT;

local db;
local isDebug = false;
local frame = CreateFrame("FRAME");

--ADDON EVENTS--;
local EVENT_ADDON_LOADED = "ADDON_LOADED";
local EVENT_PLAYER_LOGOUT = "PLAYER_LOGOUT";
local EVENT_UNIT_AURA = "UNIT_AURA";
local EVENT_PET_BATTLE_START = "PET_BATTLE_OPENING_DONE";
local EVENT_PET_BATTLE_INIT = "PET_BATTLE_OPENING_START";
local EVENT_PET_BATTLE_END = "PET_BATTLE_CLOSE";
local EVENT_HEALTH_CHANGED = "PET_BATTLE_HEALTH_CHANGED";
local EVENT_BATTLE_PET_CHANGED = "PET_BATTLE_PET_CHANGED";
local EVENT_PET_BATTLE_MAX_HEALTH_CHANGED = "PET_BATTLE_MAX_HEALTH_CHANGED";

frame:RegisterEvent(EVENT_ADDON_LOADED);
frame:RegisterEvent(EVENT_PLAYER_LOGOUT);
frame:RegisterEvent(EVENT_UNIT_AURA);
frame:RegisterEvent(EVENT_PET_BATTLE_START);
frame:RegisterEvent(EVENT_PET_BATTLE_INIT);
frame:RegisterEvent(EVENT_PET_BATTLE_END);
frame:RegisterEvent(EVENT_HEALTH_CHANGED);
frame:RegisterEvent(EVENT_BATTLE_PET_CHANGED);
frame:RegisterEvent(EVENT_PET_BATTLE_MAX_HEALTH_CHANGED);

local percentLife1 = CreateFrame("Frame", nil, UIParent);
local percentLife2 = CreateFrame("Frame", nil, UIParent);
local gameTime = CreateFrame("Frame", nil, UIParent);

function split(input, seperator)
	local t = {};
	for str in string.gmatch(input, "([^"..seperator.."]+)") do
		table.insert(t, str);
	end
	return t;
end

function frame:OnEvent(event, arg1)
	if event == EVENT_PET_BATTLE_INIT then
		UpdateGameTime();
		ShowGameTime();
    elseif event == EVENT_PET_BATTLE_START then
		UpdateUnitInfo();
		UpdateGameTime();
		ShowPercentLife();
	elseif event == EVENT_PET_BATTLE_END then
		HideUI();
	elseif event == EVENT_HEALTH_CHANGED or event == EVENT_BATTLE_PET_CHANGED or event == EVENT_PET_BATTLE_MAX_HEALTH_CHANGED then
		UpdateUnitInfo();
		ShowPercentLife();
    end
end

frame:SetScript("OnEvent", frame.OnEvent);
frame:SetScript("OnUpdate", function() UpdateGameTime(); end)

function PBI:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New(PBI_SIMPLE_NAME, defaults, true);
    db = self.db.profile;
    self:Print(PetBattleInfoPrint);
	
	local options = {
		type = 'group',
		handler = PBI,
		args = {
			debug = {
				type = "toggle",
				name = "DebugLogToggle",
				desc = "Enables/disables debug logging.",
				get = "IsDebug",
				set = "ToggleIsDebug",
			}
		}
	};
	LibStub("AceConfig-3.0"):RegisterOptionsTable("PBI", options);
	self:RegisterChatCommand("pbi", "ChatCommand");
	self:RegisterChatCommand("as", "ChatCommand");
	
	InitFrame(percentLife1, -306, -100);
	InitFrame(percentLife2, 306, -100);
	InitGameTimeFrame(gameTime, -2, -9);
	
	if C_PetBattles.IsInBattle() then
		UpdateUnitInfo();
		UpdateGameTime();
		ShowPercentLife();
		ShowGameTime();
	end
end

function InitFrame(frame, x, y)
	frame:SetWidth(1);
	frame:SetHeight(1);
	frame:SetAlpha(.90);
	frame:SetPoint("TOP", x, y);
	frame.text = frame:CreateFontString(nil,"ARTWORK");
	frame.text:SetFont("Fonts\\ARIALN.ttf", 13, "OUTLINE");
	frame.text:SetPoint("CENTER",0,0);
	frame:Hide();
end

function InitGameTimeFrame(frame, x, y)
	frame:SetWidth(155);
	frame:SetHeight(65);
	--frame:SetAlpha(.50);
	frame:SetPoint("TOP", "PetBattleFrame", "TOP", 0, 0);
	frame:CreateTexture(nil, "ARTWORK", "BattleHUD-Versus"):SetAllPoints()
	frame.text = frame:CreateFontString(nil,"ARTWORK", "GameFont_Gigantic");
	fontName, fontHeight, fontFlags = frame.text:GetFont()
	frame.text:SetFont(fontName, 17, fontFlags);
	frame.text:SetPoint("TOP", x, y);
	frame:Hide();
end

function PBI:IsDebug(info)
    return self.isDebug;
end

function PBI:ToggleIsDebug(info, value)
    self.isDebug = value;
end

function PBI:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.options);
    else
		self:Print("ChatCommand " .. input);
		LibStub("AceConfigCmd-3.0"):HandleCommand("as", "PBI", input);
    end
end

function updateDB(self, event, database)
    db = database.profile;
end

function tablelength(t)
  local count = 0;
  for _ in pairs(t) do count = count + 1 end
  return count;
end

function processUi(playerIndex)
	local petIndex = C_PetBattles.GetActivePet(playerIndex);
	if petIndex then
		local id, name, icon, maxCooldown, unparsedDescription, numTurns, petType, noStrongWeakHints = C_PetBattles.GetAbilityInfo(playerIndex, petIndex, 1);
		health = C_PetBattles.GetHealth(playerIndex, petIndex);
		maxHealth = C_PetBattles.GetMaxHealth(playerIndex, petIndex);
		if playerIndex == 1 then
			percentLife1.text:SetText(tostring(round(health/maxHealth*100, 2)).."%");
		else
			percentLife2.text:SetText(tostring(round(health/maxHealth*100, 2)).."%");
		end
	end
end

function GetFormattedGameTime()
	local hours, minutes = GetGameTime();
	return AddZeroForLessThan10(hours) .. ":" .. AddZeroForLessThan10(minutes);
end

function AddZeroForLessThan10(number)
  if(number < 10) then
    return 0 .. number
  else
    return number
 end
end

function round(x, n)
    n = math.pow(10, n or 0);
    x = x * n;
    if x >= 0 then x = math.floor(x + 0.5) else x = math.ceil(x - 0.5) end
    return x / n;
end

function ShowPercentLife()
	percentLife1:Show();
	percentLife2:Show();
end

function ShowGameTime()
	gameTime:Show();
end

function HideUI()
	percentLife1:Hide();
	percentLife2:Hide();
	gameTime:Hide();
end

function UpdateUnitInfo()
	processUi(1);
	processUi(2);
end

function UpdateGameTime()
	gameTime.text:SetText(GetFormattedGameTime());
end
